#send_as_pdf_lighbox_background {
    position: absolute;
    z-index: 10000;
    top: 0px;
    left: 0px;
    background: #000000;

    opacity: 0.75;
    filter: alpha(opacity=50);
    -ms-filter:"progid:DXImageTransform.Microsoft.Alpha(Opacity=50)";
}

#send_as_pdf_popup {
    position: absolute;
    z-index: 100001;
    width : 500px;
    background: #FFFFFF;
    padding: 20px;
}

.dont-show {
    display: none;
}